/*******************************************************************************
 * Copyright (c) 2002, 2003 Eteration Bilisim A.S., Naci Dai and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the GNU Lesser General Public License (LGPL)
 * which accompanies this distribution, and is available at
 *  http://www.gnu.org/licenses/lgpl.html
 * 
 * Contributors:
 *     Eteration Bilisim A.S. - initial API and implementation
 *     Naci M. Dai
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Eteration Bilisim A.S and Naci Dai (http://www.eteration.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Lomboz", "ObjectLearn" and "Eteration" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact info@eteration.com.
 *
 * 5. Products derived from this software may not be called "Lomboz"
 *    nor may "Lomboz" appear in their names without prior written
 *    permission of the Eteration Bilisim A.S.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Eteration Bilisim A.S.  For more
 * information on eteration, please see
 * <http://www.eteration.com/>.
 ***************************************************************************/
package org.objectweb.eclipsespeedo.drivers.xml;

import java.io.CharArrayReader;
import java.io.CharArrayWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;


import org.apache.xerces.parsers.SAXParser;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.helpers.DefaultHandler;

import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;



/**
 * Builds the config objects from their XML definitions
 * 
 * @author B. Grkem ERCAN
 */
public class ConfigBuilder extends DefaultHandler
{
	private BuildConfigInterface config = null;
	private CharArrayWriter contents = new CharArrayWriter();
	private static Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger("org.objectweb.eclipsespeedo.EclipseSpeedoPlugin");
	
	public ConfigBuilder(BuildConfigInterface buildConfig)	{
		super();
		this.config = buildConfig;
	}

	/**
	 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
	 */
	public void characters(char[] ch, int start, int length) throws SAXException {
		contents.write(ch, start, length);
	}

	/**
	 * @see org.xml.sax.ContentHandler#endElement(String, String, String)
	 */
	public void endElement(String nameSpaceURI, String localName, String qName)	throws SAXException {
		config.assesTagEnd(qName, localName, contents.toString());
	}

	/**
	 * @see org.xml.sax.ContentHandler#startElement(String, String, String, Attributes)
	 */
	public void startElement(String nameSpaceURI, String localName,	String qName, Attributes attributes) throws SAXException {
		contents.reset();
		config.assesTagStart(qName, localName,attributes);
	}

	/**
	 * Method getConfigObject.
	 * @return Object
	 */
	public Object getConfigObject()	{
		return this.config.getConfigObject();
	}
	

	
	public static DriverDefinition getDriverDefinition(String fileName) {
		try	{
			logger.log(BasicLevel.DEBUG, "Entering getDriverDefinition(), arg : "+fileName);
			return (DriverDefinition) doParse(fileName, new DriverDefinitionConfiguration());
		}
		catch (Exception e)	{
			e.printStackTrace();//J2EEPlugin.log(e);
			return null;
		}
	}


	
	private static Object doParse(String fileName, BuildConfigInterface cf)	throws	SAXNotRecognizedException,
																					SAXNotSupportedException,
																					FileNotFoundException,
																					SAXException,
																					IOException	{
		logger.log(BasicLevel.DEBUG, "Entering doParse(), arg : "+fileName);																						
		org.apache.xerces.parsers.SAXParser parser = new SAXParser();
		parser.setFeature("http://xml.org/sax/features/validation", false);
		parser.setFeature(
			"http://apache.org/xml/features/nonvalidating/load-external-dtd",
			false);
		parser.setContentHandler(new ConfigBuilder(cf));
		InputSource source = new InputSource(new FileReader(fileName));
		parser.parse(source);
		logger.log(BasicLevel.DEBUG, "Exiting doParse(), arg : "+fileName);	
		return cf.getConfigObject();
	}
	
	private static Object parseString(String content, BuildConfigInterface cf)
		throws
			SAXNotRecognizedException,
			SAXNotSupportedException,
			FileNotFoundException,
			SAXException,
			IOException
	{
		org.apache.xerces.parsers.SAXParser parser = new SAXParser();
		parser.setFeature("http://xml.org/sax/features/validation", false);
		parser.setFeature(
			"http://apache.org/xml/features/nonvalidating/load-external-dtd",
			false);
		parser.setContentHandler(new ConfigBuilder(cf));
		InputSource source = new InputSource(new CharArrayReader(content.toCharArray()));
		parser.parse(source);
		return cf.getConfigObject();
	}
	


}
