/*******************************************************************************
 * Copyright (c) 2002, 2003 Eteration Bilisim A.S., Naci Dai and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the GNU Lesser General Public License (LGPL)
 * which accompanies this distribution, and is available at
 *  http://www.gnu.org/licenses/lgpl.html
 * 
 * Contributors:
 *     Eteration Bilisim A.S. - initial API and implementation
 *     Naci M. Dai
 *     B.Gorkem ERCAN
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Eteration Bilisim A.S and Naci Dai (http://www.eteration.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Lomboz", "ObjectLearn" and "Eteration" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact info@eteration.com.
 *
 * 5. Products derived from this software may not be called "Lomboz"
 *    nor may "Lomboz" appear in their names without prior written
 *    permission of the Eteration Bilisim A.S.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Eteration Bilisim A.S.  For more
 * information on eteration, please see
 * <http://www.eteration.com/>.
 ***************************************************************************/
/*
 * Created on Mar 16, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package org.objectweb.eclipsespeedo.drivers.xml;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.objectweb.util.monolog.api.Logger;

import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;


/**
 * @author Naci Dai
 */
public class DriverDefinition  implements IPropertySource {
	

	private List fDescriptors;
	private File definitionFile;
	private String name;

	private EclipsePrefConf Eclipseprefconf;
	private JDOProjectConf JDOprojconf;


	private static Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(DriverDefinition.class.getName());
	
	public DriverDefinition() {
		Eclipseprefconf = new EclipsePrefConf();
		JDOprojconf = new JDOProjectConf();
	}

	public void addDriverClasspath(ClasspathItem cpi) {
		Eclipseprefconf.addDriverClasspath(cpi);
	}

	public void setDriverClasspath(ArrayList list) {
		Eclipseprefconf.setDriverClassPath(list);
	}
	
	public void setProjClasspath(ArrayList list) {
		Eclipseprefconf.setProjClasspath(list);
	}
	
	/**
	 * @return ArrayList
	 */
	public ArrayList getEclipsePreferencesProperties() {
		return Eclipseprefconf.getProperties();
	}
	
	public JDOProjectConfGroup getJDOProjectGroup(String name) {
		return JDOprojconf.getGroup(name);
	}
	
	public void addJDOProjectGroup(JDOProjectConfGroup grp) {
		JDOprojconf.addGroup(grp);
	}

	public DriverDefinitionProperty getPropertyNamed(String id) {
		Iterator itr = getEclipsePreferencesProperties().iterator();
		while (itr.hasNext()) {
			DriverDefinitionProperty element = (DriverDefinitionProperty) itr.next();
			if (id.equals(element.getId()))
				return element;
		}
		
		itr = JDOprojconf.getGroups().iterator();
		while (itr.hasNext()) {
			Iterator ite = ((JDOProjectConfGroup) itr.next()).getProperties().iterator();
			while (ite.hasNext()) {
				DriverDefinitionProperty element = (DriverDefinitionProperty) ite.next();
				if (id.equals(element.getId()))
					return element;
			}
		}
		return null;
	}

	/**
	 * Sets the properties.
	 * @param properties The properties to set
	 */
	public void setProperties(EclipsePrefConf epc) {
		this.Eclipseprefconf = epc;
	}

	/**
	 * @return String
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name.
	 * @param name The name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	public String toString() {
		Iterator props;
		
		String xml = "\n<driverDefinition name=\""+ getName()+ "\">\n";
		xml += Eclipseprefconf.toString();
		xml += JDOprojconf.toString();		
		xml += "\n</driverDefinition>";
		return xml;
	}

	/**
	 * @return File
	 */
	public File getDefinitionFile() {
		return definitionFile;
	}

	/**
	 * Sets the definitionFile.
	 * @param definitionFile The definitionFile to set
	 */
	public void setDefinitionFile(File definitionFile) {
		this.definitionFile = definitionFile;
	}


	/**
	 * @param str
	 * @return
	 */
	private String fixPassthroughProperties(String str) {
		String resolvedString = str;
		if (isPassPropertyLeft(resolvedString)) {
			resolvedString = fixParam(resolvedString);
			resolvedString = fixPassthroughProperties(resolvedString);
		}
		return resolvedString;
	}

	private String resolvePropertiesFromCache(
		String proppedString,
		HashMap cache) {
		String resolvedString = proppedString;
		if (isPropertyLeft(resolvedString)) {
			resolvedString = resolveProperty(resolvedString, cache);
			resolvedString = resolvePropertiesFromCache(resolvedString, cache);
		}
		return resolvedString;
	}

	private boolean isPropertyLeft(String str) {
		return str.indexOf("${") >= 0;
	}
	private boolean isPassPropertyLeft(String str) {
		return str.indexOf("%{") >= 0;
	}

	private String resolveProperty(String proppedString, HashMap cache) {
		String str = proppedString;
		int start = str.indexOf("${");
		int end = str.indexOf("}", start);
		String key = str.substring(start + 2, end);

		return str.substring(0, start)
			+ cache.get(key)
			+ str.substring(end + 1);
	}
	
	private String fixParam(String proppedString) {
		String str = proppedString;
		int start = str.indexOf("%{");
		return str.substring(0, start)
			+ "${"
			+ str.substring(start+2);
	}
	
	public Object getEditableValue()
	{
		return this;
	}

	public IPropertyDescriptor[] getPropertyDescriptors(){
		if(fDescriptors == null){
			fDescriptors = new ArrayList();
			fDescriptors.add(new PropertyDescriptor("targeted_server_name", "Name"));
			fDescriptors.add(new PropertyDescriptor("definition_file", "Definition file"));
			fDescriptors.add(new PropertyDescriptor("start_class", "Start class"));
			fDescriptors.add(new PropertyDescriptor("stop_class", "Stop class"));
			fDescriptors.add(new PropertyDescriptor("start_VM_arguments", "Start VM arguments"));
			fDescriptors.add(new PropertyDescriptor("server_home", "Server home"));
			fDescriptors.add(new PropertyDescriptor("stop_vm_arguments", "Stop VM arguments"));
			fDescriptors.add(new PropertyDescriptor("start_program_arguments", "Start program arguments"));
			fDescriptors.add(new PropertyDescriptor("stop_program_arguments", "Stop program arguments"));
			fDescriptors.add(new PropertyDescriptor("jndi_initial_context_factory", "JNDI Initial context factory"));
			fDescriptors.add(new PropertyDescriptor("jndi_url", "JNDI Provider URL"));
			fDescriptors.add(new PropertyDescriptor("start_working_dir", "Start working directory"));
			fDescriptors.add(new PropertyDescriptor("stop_working_dir", "Stop working directory"));

			for(int i = 0; i < fDescriptors.size(); i++){
				PropertyDescriptor tDesc = (PropertyDescriptor)fDescriptors.get(i);
				tDesc.setCategory("Derived server properties");
			}

			/*for(int i = 0; i < properties.size(); i++) {
				DriverDefinitionProperty p = (DriverDefinitionProperty)properties.get(i);
				PropertyDescriptor desc = new TextPropertyDescriptor(p.getId(), p.getLabel() + "(" + p.getId() + ")");
				desc.setCategory("Server definition properties");
				fDescriptors.add(desc);
			}*/
		}
		return (IPropertyDescriptor[])fDescriptors.toArray(new IPropertyDescriptor[fDescriptors.size()]);
	}

	public Object getPropertyValue(Object id)	{
		if(id.equals("definition_file"))
			return definitionFile.getPath();
		DriverDefinitionProperty prop = getPropertyNamed((String)id);
		if(prop != null)
			return prop.getDefaultValue();
		else
			return null;
	}

	public boolean isPropertySet(Object id)
	{
		return false;
	}

	public void resetPropertyValue(Object obj)	{
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {
		DriverDefinitionProperty prop = getPropertyNamed((String)id);
		int idx = Eclipseprefconf.getProperties().indexOf(prop);
		if(idx >= 0) {
			prop.setDefaultValue((String)value);
			Eclipseprefconf.getProperties().set(idx, prop);
			EclipseSpeedoPlugin.getDefault().getDriverDefinitionManager().update(this);
			return;
		}
		Iterator itr = JDOprojconf.getGroups().iterator();
		while (itr.hasNext()) {
			JDOProjectConfGroup dogrp = (JDOProjectConfGroup) itr.next();
			idx = dogrp.getProperties().indexOf(prop);
			if(idx >= 0) {
				prop.setDefaultValue((String)value);
				dogrp.getProperties().set(idx, prop);
				EclipseSpeedoPlugin.getDefault().getDriverDefinitionManager().update(this);
				return;
			}
		}
	}
	/**
	 * @return
	 */
	public EclipsePrefConf getEclipseprefconf() {
		return Eclipseprefconf;
	}

	/**
	 * @return
	 */
	public JDOProjectConf getJDOprojconf() {
		return JDOprojconf;
	}

	/**
	 * @param conf
	 */
	public void setEclipseprefconf(EclipsePrefConf conf) {
		Eclipseprefconf = conf;
	}

	/**
	 * @param conf
	 */
	public void setJDOprojconf(JDOProjectConf conf) {
		JDOprojconf = conf;
	}



}
