/*******************************************************************************
 * Copyright (c) 2002, 2003 Eteration Bilisim A.S., Naci Dai and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the GNU Lesser General Public License (LGPL)
 * which accompanies this distribution, and is available at
 *  http://www.gnu.org/licenses/lgpl.html
 * 
 * Contributors:
 *     Eteration Bilisim A.S. - initial API and implementation
 *     Naci M. Dai
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Eteration Bilisim A.S and Naci Dai (http://www.eteration.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Lomboz", "ObjectLearn" and "Eteration" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact info@eteration.com.
 *
 * 5. Products derived from this software may not be called "Lomboz"
 *    nor may "Lomboz" appear in their names without prior written
 *    permission of the Eteration Bilisim A.S.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Eteration Bilisim A.S.  For more
 * information on eteration, please see
 * <http://www.eteration.com/>.
 ***************************************************************************/
/*
 * Created on Mar 16, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package org.objectweb.eclipsespeedo.drivers.xml;

import java.util.ArrayList;

import org.objectweb.util.monolog.api.Logger;
import org.xml.sax.Attributes;

import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;

/**
 * @author Naci Dai
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class DriverDefinitionConfiguration implements BuildConfigInterface {


	static final int ECONFPOINT = 0;
	static final int PCONFPOINT = 1;
	static final int UNDEFCONFPOINT = -1;
	
	static final int DCPATH = 0;
	static final int PCPATH = 1;
	static final int UNDEFCPATH = -1;
	
	DriverDefinition currentDefinition;
	private int curConfPoint = UNDEFCONFPOINT;
	private int curCPath = UNDEFCPATH;
	private String curJDOProjGroup = "";
	private ClasspathItem currentClasspathItem;
	private ArrayList driverClasspath;
	private ArrayList projClasspath;
	private Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(DriverDefinitionConfiguration.class.getName());
	

	public void assesTagEnd(String qName, String tagName, String content) {
		if ("driverClassPath".equals(tagName)) {
			currentDefinition.setDriverClasspath(driverClasspath);
			curCPath = UNDEFCPATH;
		} else if ("projectClassPath".equals(tagName)){
			currentDefinition.setProjClasspath(projClasspath);
			curCPath = UNDEFCPATH;
		} else if ("EclipsePreferences".equals(tagName)){
			curConfPoint = UNDEFCONFPOINT;
		} else if ("JDOProject".equals(tagName)){
			curConfPoint = UNDEFCONFPOINT;
		} else if ("Group".equals(tagName)){
			curJDOProjGroup = "";
		} else if ("jar".equals(tagName)) {
			currentClasspathItem.setClasspath(content);
			if (curCPath == DCPATH){
				driverClasspath.add(currentClasspathItem);
			}
			else if (curCPath == PCPATH){
				projClasspath.add(currentClasspathItem);
			}
		}
	}


	public void assesTagStart(String qName, String tagName, Attributes attributes) {
		if ("driverDefinition".equals(tagName)) {
			currentDefinition = new DriverDefinition();
			currentDefinition.setName(attributes.getValue("name"));
		} else if ("EclipsePreferences".equals(tagName)){
			curConfPoint = ECONFPOINT;
		} else if ("JDOProject".equals(tagName)){
			curConfPoint = PCONFPOINT;
		} else if ("Group".equals(tagName)){
			curJDOProjGroup = attributes.getValue("name");
			currentDefinition.addJDOProjectGroup(new JDOProjectConfGroup(curJDOProjGroup, attributes.getValue("description")));
		} else if ("property".equals(tagName)) {
			DriverDefinitionProperty property = new DriverDefinitionProperty();
			property.setId(attributes.getValue("id"));
			property.setLabel(attributes.getValue("label"));
			property.setDefaultValue(attributes.getValue("default"));
			property.setTypeFromStr(attributes.getValue("type"));
			property.setModifiable(attributes.getValue("modifiable"));
			property.setDesc(attributes.getValue("description"));
			if (curConfPoint == ECONFPOINT){
				currentDefinition.getEclipsePreferencesProperties().add(property);
			}
			else if (curConfPoint == PCONFPOINT){
				currentDefinition.getJDOProjectGroup(curJDOProjGroup).addProperty(property);
			}
		} else if ("jar".equals(tagName)) {
			currentClasspathItem = new ClasspathItem();
			if (curCPath == DCPATH){
				currentClasspathItem.setTypeStr(attributes.getValue("type"));
			}
			else if (curCPath == PCPATH){
				currentClasspathItem.setTypeStr(attributes.getValue("type"));
			}
		} else if ("driverClassPath".equals(tagName)) {
			curCPath = DCPATH;
			driverClasspath = new ArrayList();
		} else if ("projectClassPath".equals(tagName)) {
			curCPath = PCPATH;
			projClasspath = new ArrayList();
		}
	}

	public Object getConfigObject() {
		return currentDefinition;
	}
}
