/*******************************************************************************
 * Copyright (c) 2002, 2003 Eteration Bilisim A.S., Naci Dai and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the GNU Lesser General Public License (LGPL)
 * which accompanies this distribution, and is available at
 *  http://www.gnu.org/licenses/lgpl.html
 * 
 * Contributors:
 *     Eteration Bilisim A.S. - initial API and implementation
 *     Naci M. Dai
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Eteration Bilisim A.S and Naci Dai (http://www.eteration.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Lomboz", "ObjectLearn" and "Eteration" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact info@eteration.com.
 *
 * 5. Products derived from this software may not be called "Lomboz"
 *    nor may "Lomboz" appear in their names without prior written
 *    permission of the Eteration Bilisim A.S.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Eteration Bilisim A.S.  For more
 * information on eteration, please see
 * <http://www.eteration.com/>.
 ***************************************************************************/
/*
 * Created on Mar 17, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package org.objectweb.eclipsespeedo.drivers.xml;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.runtime.Path;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;



/**
 * @author Naci Dai
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class DriverDefinitionManager {
	
	ArrayList definitions;
	File sourceDir;
	Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(this.getClass().getName());
	
	public DriverDefinitionManager(Path installUrl) {
		logger.log(BasicLevel.DEBUG, "Creating driver definition manager["+installUrl+"]");
		String s1 = installUrl.append("/drivers").toOSString();
		sourceDir = new File(s1);
		refresh();
	}
	
	
	public void refresh() {
		definitions = scanFiles(sourceDir);
	}
	
	public void update(DriverDefinition element) {
		toFile(element);
	}	
	
	public void update() {
		Iterator defs = definitions.iterator();
		while (defs.hasNext()) {
			DriverDefinition element = (DriverDefinition) defs.next();
			update(element);
		}
	}
	
	private void toFile(DriverDefinition def)	{
		try {
			def.getDefinitionFile().renameTo(new File(def.getDefinitionFile().getCanonicalFile()+".bak"));
		} catch (IOException e) {
			logger.log(BasicLevel.ERROR,e);
		}
		try {
			FileOutputStream out = new FileOutputStream(def.getDefinitionFile());
			out.write(def.toString().getBytes());
			out.close();
		} catch (FileNotFoundException e) {
			logger.log(BasicLevel.ERROR,e);
		} catch (IOException e) {
			logger.log(BasicLevel.ERROR,e);
		}
	}
	
	private  ArrayList scanFiles(File dir) {
		logger.log(BasicLevel.DEBUG, "Entering DriverDefinitionManager.ScanFiles()");
		ArrayList all = new ArrayList();
		try {
			if(dir.isDirectory()){
				File[] allDrivers = dir.listFiles(new FilenameFilter(){
					public boolean accept(File dir, String name) {
						if(name.endsWith(".driver"))
							return true;
						return false;
					}
				});
				
				for (int i = 0; i < allDrivers.length; i++) {
					File file = allDrivers[i];
					logger.log(BasicLevel.DEBUG, "Starting reading driver definition["+file+"]");
					DriverDefinition def = (DriverDefinition) ConfigBuilder.getDriverDefinition(file.getCanonicalPath());
					logger.log(BasicLevel.DEBUG, "Ending reading driver definition["+file+"]");
					if(def != null){
						def.setDefinitionFile(file);
						all.add(def);
					}
				}
			}
		} catch (IOException e) {
			logger.log(BasicLevel.ERROR,e);
		}
		catch (Exception e) {
				   logger.log(BasicLevel.ERROR,e);
		}
		return all;
	}

	/**
	 * @return ArrayList
	 */
	public ArrayList getDefinitions() {
		return definitions;
	}

	/**
	 * @return ArrayList
	 */
	public DriverDefinition getDefinitionNamed(String name) {
		refresh();
		Iterator defs = getDefinitions().iterator();
		while (defs.hasNext()) {
			DriverDefinition elem = (DriverDefinition) defs.next();
			if(name.equals(elem.getName()))
				return elem;			
		}		
		return null;
	}

	/**
	 * Sets the definitions.
	 * @param definitions The definitions to set
	 */
	public void setDefinitions(ArrayList definitions) {
		this.definitions = definitions;
	}

}
