/*******************************************************************************
 * EclipseJDO: Plugin for JDO drivers within Eclipse platform
 * Copyright (c) 2002, 2003 Versant France http://www.versant.fr
 * 
 * This program is free software; you can redistribute it and/or modify  it 
 * under the terms of the GNU General Public License as published by  the Free 
 * Software Foundation; either version 2 of the License, or  (at your option) any 
 * later version. 

 * This program is distributed in the hope that it will be useful,  but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE.  See the  GNU General Public License for more details. 

 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, write to the Free Software  Foundation, Inc., 59 Temple 
 * Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contributors:
 * 	 Versant France
 * 		Yves Yang (yang@versant.fr)
 * 
 * 	 DESS GLA - University Paris 6
 * 		- initial project
 * 		  Xiao-Ta HUANG (xthuang@dess-gla.infop6.jussieu.fr)
 * 		  Guillaume BOUICHET (gbuichet@dess-gla.infop6.jussieu.fr)  
 *******************************************************************************/
package org.objectweb.eclipsespeedo.ui.metadataeditor;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.ui.IJavaElementSearchConstants;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.dialogs.SelectionDialog;

import org.objectweb.eclipsespeedo.metadata.JDOBoolean;
import org.objectweb.eclipsespeedo.metadata.JDOIdentityType;
import org.objectweb.eclipsespeedo.metadata.JDONullValue;
import org.objectweb.eclipsespeedo.metadata.JDOPersistenceModifier;
import org.objectweb.eclipsespeedo.metadata.MetadataModelModifier;

import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;


/**
 * @author yyang
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public abstract class AbstractJDOEditor
{
	static protected String[] JDOBOOLEAN_STRINGS =
		new String[] {
			"",
			JDOBoolean.TRUE_LITERAL.getName(),
			JDOBoolean.FALSE_LITERAL.getName()};
	static protected String[] JDOCLASS_IDENTITY_TYPE_STRINGS =
		new String[] {
			"",
			JDOIdentityType.APPLICATION_LITERAL.getName(),
			JDOIdentityType.DATASTORE_LITERAL.getName(),
			JDOIdentityType.NONDURABLE_LITERAL.getName()};
	static protected String[] JDOPERSISTENCE_MODIFIER8STRING =
		new String[] {
			"",
			JDOPersistenceModifier.PERSISTENT_LITERAL.getName(),
			JDOPersistenceModifier.TRANSACTIONAL_LITERAL.getName(),
			JDOPersistenceModifier.NONE_LITERAL.getName()};
	static protected String[] JDONULLVALUE_STRINGS =
		new String[] {
			JDONullValue.NONE_LITERAL.getName(),
			JDONullValue.EXCEPTION_LITERAL.getName(),
			JDONullValue.DEFAULT_LITERAL.getName()};

	static protected String[] JDOLIST_STRINGS =
		new String[] {
			"",
			"Collection",
			"Map",
			"Array" };

	static final int FIELD_TYPE_NONE = 0;
	static final int FIELD_TYPE_COLLECTION = FIELD_TYPE_NONE + 1;
	static final int FIELD_TYPE_MAP = FIELD_TYPE_COLLECTION + 1;
	static final int FIELD_TYPE_ARRAY = FIELD_TYPE_MAP + 1;

	static protected String[] JDOLIST_STRINGS_LABEL =
		new String[] { "None", "Collection", "Map", "Array" };

	protected MetadataModelModifier modifier;
	protected IJavaProject currentJavaProject;
	boolean containerInitialize = false;
	protected Composite container;

	public AbstractJDOEditor(
		MetadataModelModifier modifier,
		IJavaProject project)
	{
		this.modifier = modifier;
		currentJavaProject = project;
	}

	protected String selectClassInProject()
	{
		Shell shell = EclipseSpeedoPlugin.getShell();
		try
		{
			SelectionDialog dialog =
				JavaUI.createTypeDialog(
					shell,
					new ProgressMonitorDialog(shell),
					SearchEngine.createJavaSearchScope(
						new IJavaElement[] { currentJavaProject }),
					IJavaElementSearchConstants.CONSIDER_TYPES,
					false);
			dialog.setTitle(EclipseSpeedoPlugin.getResourceString("MetadataComposer_typeSelection_title"));
			dialog.setMessage(EclipseSpeedoPlugin.getResourceString("MetadataComposer_typeSelection_message"));
			if (dialog.open() == IDialogConstants.CANCEL_ID)
			{
				return null;
			}

			Object[] types = dialog.getResult();
			if (types != null && types.length > 0)
			{
				IType type = (IType) types[0];
				return type.getFullyQualifiedName('.');
			}
		}
		catch (JavaModelException e)
		{
			e.printStackTrace();
		}
		return null;
	}

	public void dispose()
	{
		if (container != null && !container.isDisposed())
		{
			container.dispose();
			container = null;
		}
	}
}
