/*******************************************************************************
 * EclipseJDO: Plugin for JDO drivers within Eclipse platform
 * Copyright (c) 2002, 2003 Versant France http://www.versant.fr
 * 
 * This program is free software; you can redistribute it and/or modify  it 
 * under the terms of the GNU General Public License as published by  the Free 
 * Software Foundation; either version 2 of the License, or  (at your option) any 
 * later version. 

 * This program is distributed in the hope that it will be useful,  but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE.  See the  GNU General Public License for more details. 

 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, write to the Free Software  Foundation, Inc., 59 Temple 
 * Place, Suite 330, Boston, MA  02111-1307  USA
 *  
 * Contributors:
 * 	 Versant France
 * 		Yves Yang (yang@versant.fr)
 * 
 * 	 DESS GLA - University Paris 6
 * 		- initial project
 * 		  Xiao-Ta HUANG (xthuang@dess-gla.infop6.jussieu.fr)
 * 		  Guillaume BOUICHET (gbuichet@dess-gla.infop6.jussieu.fr)  
 *******************************************************************************/

package org.objectweb.eclipsespeedo.ui.metadataeditor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jdt.internal.ui.dialogs.StatusDialog;
import org.eclipse.jdt.internal.ui.dialogs.StatusInfo;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.ComboDialogField;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.DialogField;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.StringDialogField;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.IDialogFieldListener;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.LayoutUtil;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.metadata.JDOExtension;
import org.objectweb.eclipsespeedo.metadata.MetadataFactory;

/**
 * Dialog to enter a na new task tag
 */
public class ExtensionInputDialog extends StatusDialog {
	private class CompilerExtensionInputAdapter implements IDialogFieldListener	{
		public void dialogFieldChanged(DialogField field) {
			doValidation(field);
		}
	}

	private ComboDialogField fVendorNameDialogField;
	private ComboDialogField fKeyDialogField;
	private ComboDialogField fValueDialogField;
	private StringDialogField Value;

	private List fExistingKeys;
	private List fExistingVendorNames;

	private Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(ExtensionInputDialog.class.getName());
	private boolean canSetKeyItem = false;
	private boolean SetKeyItemDone = false;

	private boolean canSetValueItem = false;
	private boolean SetValueItemDone = false;
	private JDOExtension task; 

	public ExtensionInputDialog(
		Shell parent,
		JDOExtension task,
		List existingEntries) //if we give an Extension that means we are in editing mode otherwise we are in add mode
	{
		super(parent);
		this.task = task;
		Collection col = VendorExtensionProperties.getVendor();
		Iterator it = col.iterator();
		String v = "";
		while (it.hasNext()) {
			v = (String) it.next();
		}
		col = VendorExtensionProperties.getkeys(v);
		it = col.iterator();
		String k = "";
		while (it.hasNext())
		{
			k = (String) it.next();
		}

		col = VendorExtensionProperties.getValues(v, k);
		it = col.iterator();
		String z = "";
		while (it.hasNext())
		{
			z = (String) it.next();
		}

		fExistingVendorNames = new ArrayList(existingEntries.size());

		for (int i = 0; i < existingEntries.size(); i++) {
			JDOExtension curr = (JDOExtension) existingEntries.get(i);
			if (!curr.equals(task))	{
				fExistingVendorNames.add(curr.getVendorName());
			}
		}

		fExistingKeys = new ArrayList(existingEntries.size());

		for (int i = 0; i < existingEntries.size(); i++){
			JDOExtension curr = (JDOExtension) existingEntries.get(i);
			if (!curr.equals(task))	{
				fExistingKeys.add(curr.getKey());
			}
		}

		if (task == null)
		{
			setTitle("New extension");
		}
		else
		{
			setTitle("Edit extension"); 
		}

		//this is the listener
		CompilerExtensionInputAdapter adapter =	new CompilerExtensionInputAdapter();

		fVendorNameDialogField = new ComboDialogField(2);
		fVendorNameDialogField.setLabelText("Vendor Name"); //$NON-NLS-1$
		fVendorNameDialogField.setDialogFieldListener(adapter);
		//fVendorNameDialogField.setItems(items);
		fVendorNameDialogField.setText((task != null) ? task.getVendorName() : EclipseSpeedoPlugin.getDefault().getPreferenceStore().getString("Vendor name prefixe")); //$NON-NLS-1$
		fVendorNameDialogField.setItems(VendorExtensionProperties.getVendorArray());
		//		if (task != null)
		//		{
		//			for (int i = 0; i < items.length; i++)
		//			{
		//				if (items[i].equalsIgnoreCase(task.getVendorName()))
		//					fVendorNameDialogField.selectItem(i);
		//			}
		//		}
		//		else
		//		{
		//			fVendorNameDialogField.selectItem(0);
		//		}

		fKeyDialogField = new ComboDialogField(2);
		fKeyDialogField.setLabelText("Key");
		fKeyDialogField.setDialogFieldListener(adapter);
		fKeyDialogField.setText((task != null) ? task.getKey() : "");
		fKeyDialogField.setItems(VendorExtensionProperties.getKeysArray(EclipseSpeedoPlugin.getDefault().getPreferenceStore().getString("Vendor name prefixe")));
		canSetKeyItem = true;
		
		Value = new StringDialogField();
		Value.setLabelText("Value");
		Value.setDialogFieldListener(adapter);
		Value.setText((task != null) ? task.getValue() : "");
		canSetValueItem = true;

	}

	public JDOExtension getResult()
	{
		JDOExtension task = MetadataFactory.eINSTANCE.createJDOExtension();
		task.setVendorName(fVendorNameDialogField.getText().trim());
		task.setKey(fKeyDialogField.getText().trim());
		//task.setValue(fValueDialogField.getText().trim());
		task.setValue(Value.getText().trim());
		task.getExtensions().clear();
		return task;
	}

	protected Control createDialogArea(Composite parent)
	{
		Composite composite = (Composite) super.createDialogArea(parent);

		Composite inner = new Composite(composite, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		layout.numColumns = 2;
		inner.setLayout(layout);

		fVendorNameDialogField.doFillIntoGrid(inner, 2);
		fKeyDialogField.doFillIntoGrid(inner, 2);
		//fValueDialogField.doFillIntoGrid(inner, 2);
		Value.doFillIntoGrid(inner, 2);
		Text t = Value.getTextControl(inner);
		LayoutUtil.setHorizontalGrabbing(fKeyDialogField.getComboControl(null));
		LayoutUtil.setWidthHint(
			fKeyDialogField.getComboControl(null),
			convertWidthInCharsToPixels(45));

		fVendorNameDialogField.postSetFocusOnDialogField(parent.getDisplay());


		applyDialogFont(composite);
		return composite;
	}

	private void doValidation()
	{
		StatusInfo status = new StatusInfo();
		String testVendorName = fVendorNameDialogField.getText();

		String testKey;
		if (fKeyDialogField == null)
			testKey = "";
		else
			testKey = fKeyDialogField.getText();

		String testValue;
		if (Value == null)
			testValue = "";
		else
			testValue = Value.getText();

		if (testVendorName.length() == 0)
		{
			status.setError("Enter Name"); //$NON-NLS-1$
		}
		else
		{
			if (this.fExistingVendorNames.contains(testVendorName))
			{
				if (this.fExistingKeys.contains(testKey))
					status.setError("This Entry already Exist"); //$NON-NLS-1$
			}
			else if (
				Character.isWhitespace(testVendorName.charAt(0))
					|| Character.isWhitespace(
						testVendorName.charAt(testVendorName.length() - 1)))
			{
				status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.vendorName.noSpace")); //$NON-NLS-1$
			}

		}

		if (testKey.length() == 0)
		{
			status.setError(
				EclipseSpeedoPlugin.getResourceString(
					"ExtensionInputDialog.error.enterKey"));
		}
		else
		{
			if (this.fExistingKeys.contains(testKey))
			{
				if (this.fExistingVendorNames.contains(testVendorName))
					status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.entryExists")); //$NON-NLS-1$
			}
			else if (
				Character.isWhitespace(testKey.charAt(0))
					|| Character.isWhitespace(
						testKey.charAt(testKey.length() - 1)))
			{
				status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.key.noSpace")); //$NON-NLS-1$
			}
		}

		if (testValue.length() == 0)
		{
			status.setError(
				EclipseSpeedoPlugin.getResourceString(
					"ExtensionInputDialog.error.enterValue"));
		}
		else if (
			Character.isWhitespace(testValue.charAt(0))
				|| Character.isWhitespace(
					testValue.charAt(testValue.length() - 1)))
		{
			status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.value.noSpace")); //$NON-NLS-1$
		}

		updateStatus(status);
		if (canSetKeyItem && SetKeyItemDone == false)
		{
			SetKeyItemDone = true;
			fKeyDialogField.setItems(
				VendorExtensionProperties.getKeysArray(testVendorName));

		}

	}

	private void doValidation(DialogField diag)
	{
		StatusInfo status = new StatusInfo();
		String testVendorName = fVendorNameDialogField.getText();
		String testKey;
		if (fKeyDialogField == null)
			testKey = "";
		else
			testKey = fKeyDialogField.getText();

		String testValue;
		if (Value == null)
			testValue = "";
		else
			testValue = Value.getText();
		
		try {
			ComboDialogField tmpdiag = (ComboDialogField) diag;

			if (tmpdiag == fVendorNameDialogField)	{
				if (testVendorName.length() == 0)	{
					status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.enterName")); //$NON-NLS-1$
				}
				else {
					if (this.fExistingVendorNames.contains(testVendorName))	{
						if (this.fExistingKeys.contains(testKey))
							status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.entryExists")); //$NON-NLS-1$
					}
					else if (Character.isWhitespace(testVendorName.charAt(0))|| Character.isWhitespace(testVendorName.charAt(testVendorName.length() - 1)))	{
						status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.vendorName.noSpace")); //$NON-NLS-1$
					}
					else {
						SetKeyItemDone = false;
					}
				}
			}
	
			if (tmpdiag == fKeyDialogField)	{
				if (testKey.length() == 0) {
					status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.enterKey"));
				}
				else {
					if (this.fExistingKeys.contains(testKey)) {
						if (this.fExistingVendorNames.contains(testVendorName))	status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.entryExists")); //$NON-NLS-1$
					}
					else if (Character.isWhitespace(testKey.charAt(0))|| Character.isWhitespace(testKey.charAt(testKey.length() - 1))) {
						status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.key.noSpace"));
					}
					else {
						SetValueItemDone = false;
					}
				}
			}
		}
		catch (Exception e){
			if (testValue.length() == 0) {
				status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.enterValue"));
			}
			else if (Character.isWhitespace(testValue.charAt(0)) || Character.isWhitespace(testValue.charAt(testValue.length() - 1))) {
				status.setError(EclipseSpeedoPlugin.getResourceString("ExtensionInputDialog.error.value.noSpace"));
			}
		}



		updateStatus(status);
		if (canSetKeyItem && SetKeyItemDone == false)
		{
			SetKeyItemDone = true;
			String[] items =
				VendorExtensionProperties.getKeysArray(testVendorName);
			if (items != null)
				fKeyDialogField.setItems(items);
			else
			{
				String[] item = new String[1];
				item[0] = "";
				fKeyDialogField.setItems(item);
			}

		}

		if (canSetValueItem && SetValueItemDone == false)
		{
			SetValueItemDone = true;
			String[] items =
				VendorExtensionProperties.getValuesArray(
					testVendorName,
					testKey);
			if (items != null)
				fValueDialogField.setItems(items);
			else
			{
				String[] item = new String[1];
				item[0] = "";
				fValueDialogField.setItems(item);
			}
		}
	}
}
