package org.objectweb.speedo.generation.enhancer;

import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.CodeVisitor;
import org.objectweb.asm.Constants;
import org.objectweb.asm.Attribute;
import org.objectweb.asm.Type;
import org.objectweb.util.monolog.api.Logger;

import java.util.List;

public class MergedClassModifier extends LoggedClassAdapter {

    private List itfs;

    private String persistenceCapableSuperClass;

    private boolean fixProxyMethodOverriding;

    public MergedClassModifier(ClassVisitor classVisitor,
                               List itfs,
                               Logger logger) {
        super(classVisitor, logger);
        this.itfs = itfs;
        this.fixProxyMethodOverriding = false;
    }

    public MergedClassModifier(ClassVisitor classVisitor,
                               List itfs,
                               String persistenceCapableSuperClass,
                               Logger logger) {
        super(classVisitor, logger);
        this.itfs = itfs;
        this.fixProxyMethodOverriding = true;
        this.persistenceCapableSuperClass = persistenceCapableSuperClass;
    }

    // IMPLEMENTATION OF THE ClassVisitor INTERFACE //
    // ---------------------------------------------//

    public void visit(final int access,
                      final String name,
                      final String superName,
                      final String[] interfaces,
                      final String sourceFile) {
        // just stores the interfaces implemented by the visited class
        if (interfaces != null) {
            for (int i = 0; i < interfaces.length; ++i) {
                if (!itfs.contains(interfaces[i])) {
                    itfs.add(interfaces[i]);
                }
            }
        }
    }

    public CodeVisitor visitMethod(final int access,
                                   final String name,
                                   final String desc,
                                   final String[] exceptions,
                                   final Attribute attrs) {
        if (name.equals("<init>") || (access & Constants.ACC_ABSTRACT) != 0) {
            // do not merge constructors and abstract methods
            return null;
        } else {
	        CodeVisitor c = cv.visitMethod(access, name, desc, exceptions, attrs);
	        if (fixProxyMethodOverriding
	            && (name.equals("jdoLoadFieldsInAccessor")
	                || name.equals("jdoLoadFieldsFromAccessor")
	                || name.equals("jdoMakePersistent")
	                || name.equals("jdoLoadFields")
	                || name.equals("jdoMakeFieldPersistent"))) {
		        int off = 1;
		        c.visitVarInsn(Constants.ALOAD, 0);
		        Type[] args = Type.getArgumentTypes(desc);
		        for (int i = 0; i < args.length; ++i) {
			        c.visitVarInsn(args[i].getOpcode(Constants.ILOAD), off);
			        off += args[i].getSize();
		        }
		        c.visitMethodInsn(Constants.INVOKESPECIAL,
		            persistenceCapableSuperClass, name, desc);
	        }
	        return c;
        }
    }

    public void visitEnd() {
        // does nothing
    }
}
