/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.mivisitor;

import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class InheritanceVisitor extends AbstractMetaInfoVisitor {

	protected String getLoggerName() {
		return super.getLoggerName() + ".inheritance";
	}

	public void visitClass(SpeedoClass sc) throws SpeedoException {
		if (sc.superClassName == null || sc.superClassName.length() == 0) {
			sc.superClassName = null;
			super.visitClass(sc);
			return;
		}
		//Always use the fully qualified name
		sc.superClassName = sc.getSpeedoClassFromContext(sc.superClassName).getFQName();
		if (debug) {
			logger.log(BasicLevel.DEBUG, "class " + sc.getFQName()
			    + " extends " + sc.superClassName);
		}
		SpeedoExtension se = sc.getExtensionByKey(
				SpeedoProperties.INHERITANCE_MAPPING);
		if (se == null) {
			se = new SpeedoExtension(SpeedoProperties.VENDOR_NAME,
					SpeedoProperties.INHERITANCE_MAPPING,
					SpeedoProperties.INHERITANCE_HORIZONTAL_MAPPING, sc);
			sc.addExtension(se);
		}
		super.visitClass(sc);
	}
}
