/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */

package org.objectweb.speedo.mim.api;

import org.objectweb.jorm.api.PBinding;
import org.objectweb.perseus.persistence.api.TransactionalPersistenceManager;
import org.objectweb.perseus.cache.replacement.api.ReplaceableCacheEntry;
import org.objectweb.speedo.pm.api.ProxyManagerSwitch;
import org.objectweb.speedo.pm.api.ProxyManager;
import org.objectweb.speedo.pm.api.ProxyManagerFactory;
import org.objectweb.speedo.mim.api.SpeedoAccessor;

import javax.jdo.spi.PersistenceCapable;
import javax.jdo.PersistenceManagerFactory;
import java.util.Collection;

/**
 * This interface what is a Proxy in Speedo. A proxy a PersistentCapable
 * instance because it is the user object. It is the jorm PBinding because it is
 * unique. There are several fields: <UL>
 * <LI>The field 'isActive' indicates if the proxy is active (linked to a ProxyManager)</LI>
 * <LI>The fields 'pmf' represents the data source where
 * the current object is persistent.</LI>
 * <LI>The fields 'tpm' is the perseus TransactionalPersistenceManager managing
 * this instance.</LI>
 * </UL>
 *
 * @author	S.Chassande-Barrioz
 */
public interface SpeedoProxy
        extends PersistenceCapable, PBinding, ReplaceableCacheEntry {

    /**
     * Tests whether the Proxy infrastructure has already been set.
     * A proxy has a correct PClassMapping linked to a PBinder,
     * a PName obtained thanks to this PClassMapping.
     *
     * @return true if this object is a Proxy.
     */
    boolean jdoIsActive();

    /**
     * Sets the new value of the proxy flags
     *
     * @see #jdoIsActive()
     * @param newvalue the new value of the proxy flag
     */
    void jdoIsActive(boolean newvalue);


    /**
     * It retrieves the SpeedoAccessor instance used in the current context.
     * If the proxy is not active then the reference state is returned.
     * otherwise a $classNameFields is returned
     */
    SpeedoAccessor getSpeedoAccessor();

    SpeedoAccessor getReferenceAccessor();

    void setReferenceAccessor(SpeedoAccessor refAcc);

	SpeedoAccessor createAccessor();

	void copyAccessor(SpeedoAccessor src, SpeedoAccessor dest);

    /**
     * @return the ProxyManagerFactory which represents the data support
     * inside which the proxy is persistent.
     */
	ProxyManagerFactory jdoGetProxyManagerFactory();

    /**
     * It assignes a ProxyManagerFactory
     * @param pmf is the ProxyManagerFactory which represents the data
     * support inside which the proxy is persistent.
     */
    void jdoSetProxyManagerFactory(ProxyManagerFactory pmf);

    /**
     * @return the TransactionalPersistenceManager which manage the
     * concurrency, loading, ... of the proxy.
     */
    TransactionalPersistenceManager jdoGetTransactionalPersistenceManager();

    /**
     * It assignes a TransactionalPersistenceManager
     * @param tpm is the TransactionalPersistenceManager which manage the
     * concurrency, loading, ... of the proxy.
     */
    void jdoSetTransactionalPersistenceManager(TransactionalPersistenceManager tpm);

    /**
     * It retrieves an object helping the configuration of the Jorm class.
     * Currently Jorm is not fractalize then the returned object is the PBinder
     * class name.
     */
    Object getJormConfig();


	void jdoMakePersistent(ProxyManager pm, SpeedoAccessor sa);

    void jdoDeletePersistent(ProxyManager pm, SpeedoAccessor sa);

    /**
     * @return return the mapper name by which the instance is persistent.
     */
    String jdoGetMapperName();

    /**
     * @return return the mapper name by which the instance is persistent.
     */
    String jdoGetProjectName();

    /**
     * @return The file name of the jdo descriptor. This file must be availlable
     * in the class at runtime.
     */
    String jdoGetJdoFileName();

    /**
     * Retrieves the hints needed to build the PName of the persistent object
     * during the export action.
     */
    Object jdoGePNameHints();

    /**
     * Assignes the hints needed to build the PName of the persistent object
     * during the export action.
     */
    void jdoSetPNameHints(Object hints);

    /**
     * Adds an element in the gen class field whose number is given. This method
     * must call the speedoAdd method of the SpeedoGenClassCoherence interface,
     * or the Collection.add method (if the field contains a collection that
     * does implement the previous interface). If the field is null, a new
     * (speedo) collection must be created first.
     */
    void speedoAdd (Object elem, int gcFieldNumber);
}
