/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.naming.rdbsequence;

import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.jorm.naming.api.PExceptionExistingName;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.lib.BasicPBinder;
import org.objectweb.jorm.mapper.rdb.adapter.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.perseus.cache.api.CacheEntry;
import org.objectweb.perseus.cache.api.CacheException;
import org.objectweb.speedo.mapper.api.JormFactoryAttributes;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;

/**
 * This binder is specific to the rdb mapper.
 * @author S.Chassande-Barrioz
 */
public class RdbSequenceBinder extends BasicPBinder {

    private String seqName = null;
    private Integer seqStart = null;
    private Integer seqIncrement = null;
    private Integer seqCache = null;
    private boolean notCreated = true;
    private String nextValRequest = null;
	private Logger logger = null;

    public RdbSequenceBinder() {
        super();
        nullPName = new RdbSequencePName(this, RdbSequencePName.NULL_VALUE);
    }

	public void setMappingStructuresRule(byte mappingStructuresRule) {
		notCreated = mappingStructuresRule != JormFactoryAttributes.DO_NOTHING;
		if (logger != null && logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG,
		            "mappingStructuresRule=" + mappingStructuresRule
		            + " / notCreated=" + notCreated);
		}
	}

    public String getSequenceName() {
        return seqName;
    }

    public void setSequenceName(String seqName) {
        this.seqName = seqName;
    }

	public Integer getSeqStart() {
		return seqStart;
	}

	public void setSeqStart(Integer seqStart) {
		this.seqStart = seqStart;
	}

	public Integer getSeqIncrement() {
		return seqIncrement;
	}

	public void setSeqIncrement(Integer seqIncrement) {
		this.seqIncrement = seqIncrement;
	}

	public Integer getSeqCache() {
		return seqCache;
	}

	public void setSeqCache(Integer seqCache) {
		this.seqCache = seqCache;
	}

	public Logger getLogger() {
		return logger;
	}

	public void setLogger(Logger logger) {
		this.logger = logger;
	}

    private void createSequence(Connection conn) throws PException {
        if (seqName == null) {
            throw new PException(
                    "Impossible to allocate a new Sequence without name (null).");
        }
        RdbAdapter adapter = null;
		if (nextValRequest == null) {
			adapter = ((PMapperRdb)
					getBinderClassMapping().getPMapper()).getRdbAdapter();
			nextValRequest = adapter.getNextValInSequence(seqName);
		}
		if (notCreated) {
			synchronized (this) {
				if (notCreated) {
					if (adapter == null) {
						adapter = ((PMapperRdb)
								getBinderClassMapping().getPMapper()).getRdbAdapter();
					}
					try {
						if (!adapter.existSequence(conn, seqName)) {
							Statement s = conn.createStatement();
							s.execute(adapter.getCreateSequence(
							    seqName, seqStart, seqIncrement, seqCache));
							s.close();
						}
					} catch (SQLException e) {
						throw new PException(e,
								"Impossible to create the sequence '"
								+ seqName +"'.");
					}
					notCreated = false;
				}
			}
		}
    }

    private long allocateId(Connection conn) throws PException {
        try {
            Statement s = conn.createStatement();
            ResultSet rs = s.executeQuery(nextValRequest);
            if (rs.next()) {
                return rs.getLong(1);
            }
			rs.close();
            s.close();
            throw new PExceptionIO("Strong case: The nextval operation does " +
                    "not return any value!");
        } catch (SQLException e) {
            throw new PExceptionIO(e, "Impossible to allocate a new identifier"
                    + " on the sequence: " + seqName);
        }
    }

    // IMPLEMENTATION OF THE METHOD OF BasicPNamingContext INTERFACE //
    //---------------------------------------------------//
    public PName resolve(Object conn, PName pn) throws PException {
        if (pn == null) {
            throw new PExceptionNaming("[" + getClassName()
                    + "]: this pname is null");
        }
        if (!pn.getPNameManager().equals(this))
            throw new PExceptionNaming("[" + getClassName()
                    + "]: this pname is not valid in this binder");
        return pn;
    }

    public PName export(Object conn, Object infoitem, Object hints)
            throws PException {
		if (hints instanceof RdbSequencePName) {
			//Binder role
			PBindingCtrl pb = (PBindingCtrl) infoitem;
			byte nextstate = PStateGraph.nextStatePBinding(pb.getStatus(),
														   PBinding.ACTION_EXPORT);
			if (nextstate == PBinding.LIFECYCLE_ERROR)
				throw new PExceptionProtocol("Unauthorized operation");

			PName pn = new RdbSequencePName(this, ((RdbSequencePName) hints).value);
			if (cache != null) {
				synchronized (cache) {
					if (cache.lookup(pn) != null) {
						throw new PExceptionExistingName("[" + getClassName()
								+ "]: an object has been already export with the same identifier");
					}
					try {
						System.out.println("fix");
						cache.fix(cache.bind(pn, pb));
					} catch (CacheException e) {
						throw new PException(e, "[" + getClassName()
								+ "]: problem with cache management");
					}
				}
			}
			pb.setPName(pn);
			pb.setStatus(nextstate);
			return pn;
		} else {
			if (seqName == null && (hints instanceof String)) {
				seqName = (String) hints;
			}
        	return export(conn, infoitem);
		}
    }

    public PName export(Object c, Object en) throws PException {
        if (en == null) {
            throw new PExceptionNaming("[" + getClassName() + "]: cannot export null!");
        }
        if (en instanceof PName) {
            //Naming context role
            if (((PName) en).getPNameManager()==this) {
                return (PName) en;
            } else {
                return new RdbSequencePName(this, en);
            }
        }

        //Binder role
        PBindingCtrl pb = (PBindingCtrl) en;
        byte nextstate = PStateGraph.nextStatePBinding(pb.getStatus(),
                                                       PBinding.ACTION_EXPORT);
        if (nextstate == PBinding.LIFECYCLE_ERROR)
            throw new PExceptionProtocol("Unauthorized operation");

        Connection conn;
		if (c == null) {
			conn = (Connection) getBinderClassMapping().getPMapper().getConnection();
		} else if (c instanceof Connection) {
			conn = (Connection) c;
		} else if (c instanceof ConnectionHolder) {
			try {
				conn = (Connection) ((ConnectionHolder) c).getCHConnectionForWrite();
			} catch (PersistenceException e) {
				throw new PExceptionIO("Error during the fetching of a JDBC connection from the connection holder: " + c);
			}
		} else {
			throw new PException("Impossible to fetch a connection: unknown connection type: " + c);
		}
		createSequence(conn);

        PName pn = new RdbSequencePName(this, allocateId(conn));
        if (c == null) {
            getBinderClassMapping().getPMapper().closeConnection(conn);
        }
        if (cache != null) {
            synchronized (cache) {
                if (cache.lookup(pn) != null) {
                    throw new PExceptionExistingName("[" + getClassName()
                            + "]: an object has been already export with the same identifier");
                }
                try {
                    cache.fix(cache.bind(pn, pb));
                } catch (CacheException e) {
                    throw new PException(e, "[" + getClassName()
                            + "]: problem with cache management");
                }
            }
        }
        pb.setPName(pn);
        pb.setStatus(nextstate);
        return pn;
    }

    public void unexport(Object conn, PName pn) throws PException {
        if (pn.isNull()) {
            throw new PExceptionProtocol("[" + getClassName() + "]: cannot unexport with a null pname.");
        }
        if (cache != null) {
            PBindingCtrl pb = null;
            CacheEntry ce = cache.lookup(pn);
            if (ce != null) {
                pb = (PBindingCtrl) ce.getCeObject();
            }
            if (pb == null) {
                throw new PExceptionProtocol("Unauthorized operation: No Pbinding found");
            }
            byte nextstate = PStateGraph.nextStatePBinding(
                pb.getStatus(), PBinding.ACTION_UNEXPORT);
            if (nextstate == PBinding.LIFECYCLE_ERROR) {
                throw new PExceptionProtocol("Unauthorized operation: status =" + pb.getStatus());
            }
            pb.setStatus(nextstate);
        } else {
            throw new PExceptionProtocol("Unauthorized operation: No Pbinding can be found without a CacheManager");
        }
    }

    public void unexport(Object conn, PName pn, Object hints) throws PException {
        if (pn.isNull()) {
            throw new PExceptionProtocol("[" + getClassName() + "]: cannot unexport with a null pname.");
        }
        PBindingCtrl pb = null;
        if (hints instanceof PBindingCtrl) {
            pb = (PBindingCtrl) hints;
        } else if (cache != null) {
            CacheEntry ce = cache.lookup(pn);
            if (ce != null) {
                pb = (PBindingCtrl) ce.getCeObject();
            }
        }
        if (pb == null) {
            throw new PExceptionProtocol("Unauthorized operation: No Pbinding found");
        }
        byte nextstate = PStateGraph.nextStatePBinding(
            pb.getStatus(), PBinding.ACTION_UNEXPORT);
        if (nextstate == PBinding.LIFECYCLE_ERROR) {
            throw new PExceptionProtocol("Unauthorized operation: status =" + pb.getStatus());
        }
        pb.setStatus(nextstate);
    }

    public long encodeLong(PName pn) throws PExceptionNaming, UnsupportedOperationException {
        return ((RdbSequencePName) pn).value;
    }

    public PName decodeLong(long en) throws PExceptionNaming, UnsupportedOperationException {
        return new RdbSequencePName(this, en);
    }

    public PName decodeString(String en) throws PExceptionNaming {
        try {
            return new RdbSequencePName(this, Long.parseLong(en));
        } catch (NumberFormatException e) {
            throw new PExceptionNaming("[" + getClassName()
                    + "]: Impossible to decode the String to long: " + en);
        }
    }

    public boolean codingSupported(int codingtype) {
        switch(codingtype) {
            case CTLONG:
            case CTSTRING:
                return true;
            default:
                return false;
        }
    }

    public String encodeString(PName pn) throws PExceptionNaming {
        return "" + ((RdbSequencePName) pn).value;
    }
}
