/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: X. Spengler
 *
 */

package org.objectweb.speedo.query.lib;

import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.lib.JormPathHelper;
import org.objectweb.medor.api.EvaluationException;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.eval.api.QueryEvaluator;
import org.objectweb.medor.eval.lib.MedorEvaluator;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.optim.api.QueryTransformer;
import org.objectweb.medor.optim.jorm.JormFlatten2Rdb;
import org.objectweb.medor.optim.lib.IndexesGenerator;
import org.objectweb.medor.optim.lib.FlattenQueryTreeRule;
import org.objectweb.medor.optim.lib.BasicQueryRewriter;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.query.api.OrderField;
import org.objectweb.medor.query.jorm.api.JormExtent;
import org.objectweb.medor.query.jorm.lib.JormQueryTreeHelper;
import org.objectweb.medor.query.lib.QueryTreePrinter;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.query.lib.Project;
import org.objectweb.medor.query.lib.BasicOrderField;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.type.lib.QType;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.perseus.persistence.api.StateFilter;
import org.objectweb.perseus.persistence.api.State;
import org.objectweb.perseus.cache.replacement.api.ReplaceableCacheEntry;
import org.objectweb.perseus.cache.api.UnFixProtocolException;
import org.objectweb.speedo.mapper.api.JormFactory;
import org.objectweb.speedo.mapper.lib.DelegatePMapper;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.pm.api.ProxyManager;
import org.objectweb.speedo.mim.api.SpeedoProxy;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.query.api.CompiledQuery;
import org.objectweb.speedo.query.api.QueryDefinition;
import org.objectweb.speedo.query.parser.SpeedoQLQueryFilterVisitor;
import org.objectweb.speedo.query.parser.ASTSpeedoQL;
import org.objectweb.speedo.query.parser.SpeedoQLVariableVisitor;
import org.objectweb.speedo.query.parser.SpeedoQL;
import org.objectweb.speedo.query.parser.SimpleNode;
import org.objectweb.speedo.query.parser.ParseException;
import org.objectweb.speedo.workingset.api.Transaction;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import javax.jdo.JDOException;
import javax.jdo.JDOUserException;
import javax.jdo.JDOFatalException;
import java.io.CharArrayReader;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import java.util.Collections;

/**
 * SpeedoCompiledQuery object represents a query. This object is created
 * when a new query is created, and can be used several times. A list of
 * SpeedoCompiledQuery is managed with a SpeedoQueryManager object.
 * When a user creates a new JDO Query object (SpeedoQuery), a
 * SpeedoCompiledQuery object is associated to the SpeedoQuery object which is
 * used to delegate some methods.
 */
public class SpeedoCompiledQuery
        implements CompiledQuery, StateFilter, ReplaceableCacheEntry {

    private short status;
	private ClassLoader classLoader;

    /**
     * Fields from javax.jdo.Query
     */
	private QueryDefinitionImpl qd = null;

    /**
     * vparams, hparams and hvars are internal objects to manipulate the list
     * of parameters, and the list of variables.
     */
    private List vparams = null;
    private Map hparams = null;
    private Map paramName2paramClass = null;
    private Map hvars = null;
    private QueryEvaluator evaluator = null;

    /**
     * The query tree which is the result from the parsing of the given filter
     */
    private QueryTree qt = null;

    private Collection pncParams = null;
    /**
     * Logger for monolog
     */
    private Logger logger = null;
	private Logger varParserlogger = null;
	private Logger filterParserLogger = null;

    private DelegatePMapper mapper = null;
    private JormFactory jf;


    public static boolean prefetchResult = true;
    private PClassMapping pcm;
    private int identifierIndex = -1;

	private long age = 0;
	private int fixCount = 0;


    public void init(Logger l,
					 Logger logParserVar,
					 Logger logParserFil,
					 PMapper m,
					 JormFactory jf) {
        logger = l;
		varParserlogger = logParserVar;
		filterParserLogger = logParserFil;
        mapper = new DelegatePMapper(m);
		this.jf = jf;
        mapper.setJormFactory(jf);
    }

    public PMapper getMapper() {
        return mapper;
    }

    public void setMapper(PMapper m) {
        mapper = new DelegatePMapper(m);
        if (jf != null) {
            mapper.setJormFactory(jf);
        }
    }

    public void setJormFactory(JormFactory jf) {
        this.jf = jf;
        if (mapper != null) {
            mapper.setJormFactory(jf);
        }
    }

    public void defineQuery(QueryDefinition _qd) {
	    this.qd = new QueryDefinitionImpl(_qd);
        classLoader = _qd.getCandidateClass().getClassLoader();
        if (classLoader == null) {
            classLoader = getClass().getClassLoader();
			if (classLoader == null) {
				classLoader = ClassLoader.getSystemClassLoader();
				logger.log(BasicLevel.DEBUG, "The system classLoader of the " +
						"class is assigned to the query: " + classLoader);
			} else {
				logger.log(BasicLevel.DEBUG, "The classLoader of Speedo is" +
						" assigned the query: " + classLoader);
			}
		} else {
			logger.log(BasicLevel.DEBUG, "The classLoader of the class is " +
					"assigned to the query: " + classLoader);
		}
        mapper.setClassLoader(classLoader);
        this.qd.parameters = _qd.getParameters();
        this.qd.variables = _qd.getVariables();
        this.qd.filter = _qd.getFilter();
        if (_qd.isCollection())
            this.qd.candidateInstances = _qd.getCollection();
        else
            this.qd.extentClass = _qd.getExtent();
        this.qd.ignoreCache = _qd.isIgnoreCache();
        this.qd.order = _qd.getOrder();
        status = DEFINED;
    }

    // IMPLEMENTATION OF ReplaceableCacheEntry INTERFACE //
    //---------------------------------------------------//
	public long getCeAge() {
		return age;
	}

	public void setCeAge(long age) {
		this.age = age;
	}

	public void fixCe() {
		fixCount++;
	}

	public void unfixCe() throws UnFixProtocolException {
		fixCount--;
	}

	public int getCeFixCount() {
		return fixCount;
	}

	public Object getCeObject() {
		return this;
	}

	public Object getCeIdentifier() {
		return qd;
	}

    // IMPLEMENTATION OF CompiledQuery INTERFACE //
    //-------------------------------------------//

    /**
     * compile the current SpeedoCompiledQuery.
     * The query is prepared to be executed.
     * The PersistenceManager is set (even if there was a previous definition
     * of a PersistenceManager.
     */
    public synchronized void compile() throws SpeedoException, MedorException, ExpressionException {
        if (status == UNDEFINED)
            throw new SpeedoException("Impossible to compile an undefined query");
        if (status == COMPILED)
            return;
        boolean debug = logger.isLoggable(BasicLevel.DEBUG);
        // create a speedoQL object with a filter string
	    String filter = qd.filter;
/*
	    if (filter !=null && filter.length() > 0) {
		    boolean addParenthesis = filter.charAt(0) != '(';
		    if (!addParenthesis) {
				//The filter is not parenthesed
				StringTokenizer st = new StringTokenizer(filter, "()", true);
				int open = 0;
				while (st.hasMoreTokens()) {
					String token = st.nextToken();
					char c = token.charAt(0);
					switch(c) {
					case '(':
						open ++;
						break;
					case ')':
						open --;
						break;
					}
					if (open == 0) {
						break;
					}
				}
				addParenthesis = st.hasMoreTokens();
		    }
		    if (addParenthesis) {
			    //insert parenthesis around the filter
		    }
	    }
*/
	    filter = '(' + filter + ')';
        SimpleNode node = null;
        try {
            node = new SpeedoQL(new CharArrayReader(filter.toCharArray())).SpeedoQL();
        } catch (ParseException e) {
            throw new SpeedoException(
                    "Impossible to parse the filter and to create AST", e);
        }

        Manager miManager = mapper.getMetaInfoManager();
        if (miManager == null)
            throw new SpeedoException(
                    "A non null Meta information manager is needed");

        // create a new SpeedoQLVariableVisitor object with the previous
        // node and a Schema (jorm metainformation) array
        SpeedoQLVariableVisitor sqvv = new SpeedoQLVariableVisitor(node, mapper);
        sqvv.setLogger(varParserlogger);

        // create representations of the parameters list and the variable
        // list
        toHashtableParams(qd.parameters, ",");
        toHashtableVars(qd.variables, ";");

        // set these representations to the SpeedoQLVariableVisitor object
        sqvv.setParams(hparams);
        sqvv.setVars(hvars);
        sqvv.setOrders(qd.order);

        // set up the nameof the current class
        // could it be null ??? to verify it ... !!!
        sqvv.setCurrentClass(qd.candidateClass.getName());

        // start the variable visitor to catch all variables an build a
        // first tree of them
        try {
            sqvv.startVisiting();
        } catch (Exception e) {
            throw new SpeedoException(
                    "Impossible to visit the variable of the filter", e);
        }

        // get the query tree to set up the query filter visitor
        qt = sqvv.getQueryTree();

        if (debug) {
            Field[] fields = qt.getTupleStructure().getFields();
            for (int i = 0; i < fields.length; i++) {
                logger.log(BasicLevel.DEBUG,
                        "i=" + i + ", fieldname=<" + fields[i].getName() + ">");
            }
        }

        //Assign mappers and project and project name on the JormExtent nodes
        Collection extents = JormQueryTreeHelper.getJormExtents(qt);
        if (logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG, "Extent nodes: " + extents.size());
        try {
            for (Iterator it = extents.iterator(); it.hasNext();) {
                JormExtent je = (JormExtent) it.next();
                if (debug)
                    logger.log(BasicLevel.DEBUG, "Looking the extent of the class " + je.getJormName());
                PClassMapping pcm = jf.getPClassMapping(
                        JormPathHelper.getOriginClass(je.getJormName()),
                        classLoader);
                if (debug)
                    logger.log(BasicLevel.DEBUG,
                            "JormExtent: " + je + " / pcm=" + pcm);
                je.setPMapper(pcm.getPMapper(), pcm.getProjectName());
            }
        } catch (PException e) {
            throw new SpeedoException("Error while fetching PClassPMapping", e);
        }

        // Build the root QueryNode which defines the filter and the projected
        // fields
        QueryNode root;
        if (!filter.equals("(true)") && !filter.equals("true")) {
            root = new SelectProject("", null);
            if (debug) {
                logger.log(BasicLevel.DEBUG, "filter = " + qd.filter);
            }
            SpeedoQLQueryFilterVisitor qfilter =
                    new SpeedoQLQueryFilterVisitor(qt, (ASTSpeedoQL) node);

			qfilter.setLogger(filterParserLogger);

            // set the representations of the variables and parameters to the
            // SpeedoQLQueryFilterVisitor object
            qfilter.setParams(hparams);
            qfilter.setVars(hvars);

            // set up the nameof the current class
            // could it be null ??? to verify it ... !!!
            qfilter.setCurrentClass(qd.candidateClass.getName());

            // start the query filter visitor, to build and expression tree of
            // the filter expression
            try {
                qfilter.startVisiting();
            } catch (Exception e) {
                throw new SpeedoException(
                        "Impossible to built the queryTree during the filter visit", e);
            }

            // this expression is the filter of the SelectProject
            Expression exprFilter = qfilter.getQueryFilter();
            if (debug) {
                logger.log(BasicLevel.DEBUG, "filter = " + exprFilter.getClass().getName());
            }
            root.setQueryFilter(exprFilter);
        } else {
            root = new Project("", null);
        }

        if (prefetchResult) {
            JormQueryTreeHelper.addPrefetchFields(sqvv.classExtent, qt, root);
            pcm = mapper.lookup(qd.candidateClass.getName());
            identifierIndex = root.getTupleStructure().getSize() + 1;
        }

        // Project the pname
        QueryTreeField qtf = (QueryTreeField)
                qt.getTupleStructure().getField(
                        qd.candidateClass.getName() + "." + Field.PNAMENAME);
        root.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});

        if (qd.order != null && qd.order.size() > 0) {
            OrderField[] ofs = new OrderField[qd.order.size()];
            for(int i=0; i<ofs.length; i++) {
                String o = (String) qd.order.get(i);
                int idx = o.indexOf(' ');
                boolean desc = false;
                if (idx != -1) {
                    desc = o.substring(idx + 1).trim().equals("descending");
                    o = o.substring(0, idx);
                }
                o = qd.candidateClass.getName() + '.' + o;
                ofs[i] = new BasicOrderField((QueryTreeField)
                    qt.getTupleStructure().getField(o), desc);
            }
            root.setOrderBy(ofs);
        }
        qt = root;

        logger.log(BasicLevel.INFO, "QueryTree built");
        if (debug) {
            QueryTreePrinter.printQueryTree(qt, logger);
        }


        // Optimize the medor QueryTree
        //-----------------------------
//TODO: remove the dependency to JormFlatten2Rdb
        ArrayList rules = new ArrayList(2);
        rules.add(new FlattenQueryTreeRule());
        rules.add(new JormFlatten2Rdb());
        QueryTransformer queryTransformer = new BasicQueryRewriter(rules);

        IndexesGenerator indexesGenerator = new IndexesGenerator();
        try {
            qt = queryTransformer.transform(qt);
            if (debug) {
                logger.log(BasicLevel.DEBUG, "Query optimized");
                QueryTreePrinter.printQueryTree(qt, logger);
            }
            qt = indexesGenerator.transform(qt);
        } catch (Exception e) {
            throw new MedorException("Impossible to optimize the query", e);
        }

        pncParams = JormQueryTreeHelper.getRequiredPNameManagers(qt);
        for (Iterator it = pncParams.iterator(); it.hasNext();) {
            ParameterOperand po = (ParameterOperand) it.next();
            String name = po.getName();
            if (debug) {
                logger.log(BasicLevel.DEBUG, "ParameterOperand " + name + " is expected.");
            }
            po.setValue(JormPathHelper.getPNameCoder(name, mapper));
        }

        // Creates an evaluator associated to the QueryTree
        evaluator = new MedorEvaluator(qt, 0);
        status = COMPILED;
        logger.log(BasicLevel.INFO, "Query compiled");
    }

    /**
     * evaluate the query with a single parameter which is a array of object
     * parameters.
     * @param a    the array parameter of the query
     * @param pm    the persistence manager object
     * @return  a Collection of result objects
     * @throws org.objectweb.medor.api.EvaluationException
     * @throws org.objectweb.medor.api.MedorException
     */
    public Collection execute(Object[] a, ProxyManager  pm)
            throws SpeedoException, MedorException, ExpressionException {
        if (status != COMPILED)
            throw new EvaluationException(
                    "Impossible to execute a query if it has not been defined and compiled before");
        ParameterOperand[] pos = null;
        int i = 0;
        if (a != null) {
            pos = new ParameterOperand[a.length + pncParams.size()];
            for (;i < a.length; i++) {
                pos[i] = new BasicParameterOperand(
                    (ParameterOperand) hparams.get(vparams.get(i)));
                if (pos[i].getType().getTypeCode() == QType.TYPECODE_PNAME) {
                    //Assign the pname of the parameter
                    if (a[i] == null) {
	                    try {
		                    pos[i].setValue(
		                        jf.getPClassMapping(
		                        (Class) paramName2paramClass.get(pos[i].getName()))
		                        .getPBinder().getNull());
	                    } catch (Exception e) {
		                    throw new SpeedoException(
		                        "Impossible to find the null PName of the parameter (name= "
		                        + pos[i].getName()
		                        + " / type= " + pos[i].getType().getJormName(),
		                        e);
	                    }
                    } else if (!(a[i] instanceof SpeedoProxy)) {
                        throw new JDOUserException("The parameter '"
                                + pos[i].getName()
                                + "' must be a persistent object: " + a[i]);
                    } else {
                        pos[i].setValue(((SpeedoProxy) a[i]).getPName());
                    }
                } else {
                    pos[i].setValue(a[i]);
                }
            }
        } else {
            pos = new ParameterOperand[pncParams.size()];
        }
        for (Iterator it = pncParams.iterator(); it.hasNext();i++) {
            pos[i] = (ParameterOperand) it.next();
        }
        return executeQT(pm, pos);
    }


    /**
     * evaluate the query with a single parameter which is a Map of object parameters.
     * @param m    the map parameter of the query
     * @param pm    the persistence manager object
     * @return  a Collection of result objects
     * @throws org.objectweb.medor.api.EvaluationException
     * @throws org.objectweb.medor.api.MedorException
     */
    public Collection execute(Map m, ProxyManager  pm)
            throws SpeedoException, MedorException, ExpressionException {
        if (status != COMPILED)
            throw new EvaluationException(
                    "Impossible to execute a query if it has not been defined and compiled before");
        ParameterOperand[] pos =
                new BasicParameterOperand[m.size() + +pncParams.size()];
        int i = 0;
	    for (; i < m.size(); i++) {
		    pos[i] = (BasicParameterOperand) hparams.get(vparams.get(i));
		    if (pos[i].getType().getTypeCode() == QType.TYPECODE_PNAME) {
			    //Assign the pname of the parameter
			    Object o = m.get(pos[i].getName());
			    if (o == null) {
				    try {
					    pos[i].setValue(
					        jf.getPClassMapping(
					        (Class) paramName2paramClass.get(pos[i].getName()))
					        .getPBinder().getNull());
				    } catch (Exception e) {
					    throw new SpeedoException(
					        "Impossible to find the null PName of the parameter (name= "
					        + pos[i].getName()
					        + " / type= " + pos[i].getType().getJormName(),
					        e);
				    }
			    } else if (!(o instanceof SpeedoProxy)) {
				    throw new JDOUserException("The parameter '"
				        + pos[i].getName()
				        + "' must be a persistent object: " + o);
			    } else {
				    pos[i].setValue(((SpeedoProxy) o).getPName());
			    }
		    } else {
			    pos[i].setValue(m.get(pos[i].getName()));
		    }
	    }
        for (Iterator it = pncParams.iterator(); it.hasNext();) {
            pos[i++] = (ParameterOperand) it.next();
        }
        return executeQT(pm, pos);
    }

    public short getStatus() {
        return status;
    }

    // Private Methods //
    //-----------------//

    /**
     * Hash a String, and compute a Hashtable
     * example:
     *   ("String name, Float salary, Employee boss", ",")
     *   keys     | values
     *   ---------------------
     *   "name"   | "String"
     *   "salary" | "Float"
     *   "boss"   | "Employee"
     *
     * @param stringToHash	the String to hash
     * @param separator		the separator to tokenize
     */
    private void toHashtableParams(String stringToHash, String separator) {
        hparams = new HashMap();
	    paramName2paramClass = new HashMap();
        if (stringToHash != null) {
            StringTokenizer tok = new StringTokenizer(stringToHash, separator);
            vparams = new ArrayList(tok.countTokens());
            while (tok.hasMoreTokens()) {
                String tuple = tok.nextToken().trim();
                int i = tuple.indexOf(' ');
                String name = tuple.substring(i + 1, tuple.length());
	            String paramType = tuple.substring(0, i).trim();
	            PType type = getPType(paramType);
                hparams.put(name, new BasicParameterOperand(
                        type, name));
                vparams.add(name);
	            if (type == PTypeSpaceMedor.PNAME) {
	                paramName2paramClass.put(name, getClass(paramType));
                }
            }
        }
    }

    private void toHashtableVars(String stringToHash, String separator) {
        hvars = new HashMap();
        if (stringToHash != null) {
            StringTokenizer tok =
                    new StringTokenizer(stringToHash, separator);
            while (tok.hasMoreTokens()) {
                String tuple = tok.nextToken().trim();
                int i = tuple.indexOf(' ');
                String type = tuple.substring(0, i);
                hvars.put(tuple.substring(i + 1, tuple.length()), type);
            }
        }
    }

	private PType getPType(String name) {
		if ("String".equals(name)) {
			return PTypeSpace.STRING;
		}
		if ("Integer".equals(name)) {
			return PTypeSpace.OBJINT;
		}
		for (int i = 0; i < PTypeSpace.PREDEFINEDPTYPES.length; i++) {
			PType type = PTypeSpace.PREDEFINEDPTYPES[i];
			if (type.getJavaName().equals(name)
			    || type.getJormName().equals(name)) {
				return type;
			}
		}
		return PTypeSpaceMedor.PNAME;
	}
    private Class getClass(String classname) {
        if (classLoader != null) {
            try {
                return classLoader.loadClass(classname);
            } catch (ClassNotFoundException e) {
                try {
                    return classLoader.loadClass(
                            qd.candidateClass.getPackage().getName() + "." + classname);
                } catch (ClassNotFoundException e2) {
                    return null;
                }
            }
        } else {
            return null;
        }
    }

    /**
     * executes a the current query, and returns a Collection of object.
     * @return  a new Collection of objects.
     * @throws org.objectweb.medor.api.EvaluationException
     * @throws org.objectweb.medor.api.MedorException
     */
	private Collection executeQT(ProxyManager pm, ParameterOperand[] pos)
        throws EvaluationException, MedorException {
		if (!qd.ignoreCache || !pm.getIgnoreCache()) {
			try {
				pm.getTransactionalPersistenceManager()
						.flush((Transaction) pm.currentTransaction(), this);
			} catch (PersistenceException e) {
				throw new JDOException("Impossible to use the cache into a " +
						"query: Error during the flushing of data on the support", e);
			}
		}
        // Calculates and gets the required connection ressources for this query
        ConnectionResources connRess =
                evaluator.getRequiredConnectionResources();

        // Gets the QueryLeafs that require connections
        QueryLeaf[] leafs = connRess.getRequiredQueryLeafConnection();
        Object connection = ((Transaction) pm.currentTransaction())
				.getConnectionHolder();

        // Setting QueryLeaf's appropriated connection Object
        for (int cpt = 0; (cpt < leafs.length); cpt++) {
            connRess.setConnection(leafs[cpt], connection);
        }
        PrefetchBuffer prefetchBuffer = null;
        if (prefetchResult) {
            prefetchBuffer = mapper.getPrefetchCache().createPrefetchBuffer(
					pcm, pm.currentTransaction(),
					identifierIndex,
					pm.getMultithreaded());
        }
        // Launching Medor Evaluator...
	    if (logger.isLoggable(BasicLevel.DEBUG)) {
		    logger.log(BasicLevel.DEBUG, "Parameters: " + posToString(pos));
	    }
        TupleCollection queryResult = null;
		try {
			queryResult = evaluator.evaluate(pos, connRess, prefetchBuffer);
		} catch (EvaluationException e) {
			throw new JDOFatalException(
					"Impossible to evaluate the query: "
					+ ExceptionHelper.getNested(e)
			);
		}
		if (queryResult == null || queryResult.isEmpty()) {
			logger.log(BasicLevel.INFO,
					"Be careful, the result collection is empty");
			return Collections.EMPTY_SET;
		}
		return new QueryResultCollection(
				queryResult, pm, new Object[]{connection}, logger);
    }

	// IMPLEMENTATION OF THE CacheEntryFilter INTERFACE //
	//--------------------------------------------------//

	public boolean accept(State ce) {
		return true;
	}


	private String posToString(ParameterOperand[] pos) {
		if (pos == null) {
			return "null";
		}
		StringBuffer sb = new StringBuffer("[");
		String sep = "";
		for(int i=0; i<pos.length; i++) {
			sb.append(sep);
			sb.append("(name='");
			sb.append("'");
			sb.append(pos[i].getName());
			sb.append("', value=");
			sb.append(pos[i]);
			sb.append(")");
		}
		sb.append("]");
		return sb.toString();
	}
}